"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPackageManagerCommand = exports.PM = exports.detectPackageManager = void 0;
exports.getCollectorByPackageManager = getCollectorByPackageManager;
exports.getNodeModules = getNodeModules;
const npmNodeModulesCollector_1 = require("./npmNodeModulesCollector");
const pnpmNodeModulesCollector_1 = require("./pnpmNodeModulesCollector");
const yarnNodeModulesCollector_1 = require("./yarnNodeModulesCollector");
const packageManager_1 = require("./packageManager");
Object.defineProperty(exports, "detectPackageManager", { enumerable: true, get: function () { return packageManager_1.detectPackageManager; } });
Object.defineProperty(exports, "PM", { enumerable: true, get: function () { return packageManager_1.PM; } });
Object.defineProperty(exports, "getPackageManagerCommand", { enumerable: true, get: function () { return packageManager_1.getPackageManagerCommand; } });
async function getCollectorByPackageManager(rootDir, tempDirManager) {
    const manager = (0, packageManager_1.detectPackageManager)(rootDir);
    switch (manager) {
        case packageManager_1.PM.PNPM:
            if (await pnpmNodeModulesCollector_1.PnpmNodeModulesCollector.isPnpmProjectHoisted(rootDir)) {
                return new npmNodeModulesCollector_1.NpmNodeModulesCollector(rootDir, tempDirManager);
            }
            return new pnpmNodeModulesCollector_1.PnpmNodeModulesCollector(rootDir, tempDirManager);
        case packageManager_1.PM.NPM:
            return new npmNodeModulesCollector_1.NpmNodeModulesCollector(rootDir, tempDirManager);
        case packageManager_1.PM.YARN:
            return new yarnNodeModulesCollector_1.YarnNodeModulesCollector(rootDir, tempDirManager);
        default:
            return new npmNodeModulesCollector_1.NpmNodeModulesCollector(rootDir, tempDirManager);
    }
}
async function getNodeModules(rootDir, tempDirManager) {
    const collector = await getCollectorByPackageManager(rootDir, tempDirManager);
    return collector.getNodeModules();
}
//# sourceMappingURL=index.js.map